Index: lisp.el
===================================================================
RCS file: /usr/CVSroot/XEmacs/xemacs/lisp/lisp.el,v
retrieving revision 1.4.2.2
diff -u -r1.4.2.2 lisp.el
--- lisp.el	2000/11/23 06:27:37	1.4.2.2
+++ lisp.el	2001/02/27 01:35:58
@@ -169,18 +169,63 @@
   (and (beginning-of-defun-raw arg)
        (progn (beginning-of-line) t)))
 
+;; XEmacs change:
+;; Use parse-partial-sexp and scan-lists to properly deal with syntax,
+;; rather than using regexp's to find defun beginnings. Fixes the ^(
+;; embedded in a comment/string bug.
 (defun beginning-of-defun-raw (&optional arg)
   "Move point to the character that starts a defun.
 This is identical to beginning-of-defun, except that point does not move
 to the beginning of the line when `defun-prompt-regexp' is non-nil."
   (interactive "p")
   (and arg (< arg 0) (not (eobp)) (forward-char 1))
-  (and (re-search-backward (if defun-prompt-regexp
-			       (concat "^\\s(\\|"
-				       "\\(" defun-prompt-regexp "\\)\\s(")
-			     "^\\s(")
-			   nil 'move (or arg 1))
-       (progn (goto-char (1- (match-end 0)))) t))
+
+  (if (eq arg 0)
+      t
+    (let* ((arg (or arg 1))
+	   (state (parse-partial-sexp (point-min) (point)))
+	   (depth (nth 0 state))
+	   (pt (if (or (nth 3 state) (nth 4 state))
+		   (nth 8 state)
+		 (point)))
+	   (dir (if (> arg 0)
+		    -1
+		  1))
+	   p)
+	
+      (cond ((> depth 0)
+	     (setq pt (scan-lists pt dir depth)))
+	    ((eq dir -1)
+	     (if (setq p (scan-lists pt dir 0))
+		 (setq pt p)
+	       (setq pt (buffer-end dir)))))
+
+      (while (not (or (eq arg 0)
+		      (eq pt (buffer-end dir))))
+
+	(when (eq dir 1)
+	  (setq state (parse-partial-sexp pt (point-max) 1))
+	  (setq pt (if (eq 1 (nth 0 state))
+		       (1- (point))
+		     (point))))
+
+	(goto-char pt)
+	(beginning-of-line)
+	(if (and (looking-at (if defun-prompt-regexp
+				 (concat "^\\s(\\|"
+					 "\\("
+					 defun-prompt-regexp
+					 "\\)\\s(")
+			       "^\\s("))
+		 (eq pt (1- (match-end 0))))
+	    (setq arg (+ arg dir)))
+	  
+	(if (not (eq arg 0))
+	    (if (setq p (scan-lists pt dir 0))
+		(setq pt p)
+	      (setq pt (buffer-end dir)))))
+      (goto-char pt)
+      (eq arg 0))))
 
 ;; XEmacs change (optional buffer parameter)
 (defun buffer-end (arg &optional buffer)
